@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: bacon-video-widget-gst.c,v 1.68 2004/02/13 10:34:41 dolphy Exp $
+ * $Id: bacon-video-widget-gst.c,v 1.79 2004/04/05 16:57:16 dolphy Exp $
  *
  */
 
@@ -150,9 +150,10 @@ struct BaconVideoWidgetPrivate
 };
 
 enum {
-  VIDEO_SIZE,
-  GST_ERROR,
-  FOUND_TAG
+  ASYNC_VIDEO_SIZE,
+  ASYNC_ERROR,
+  ASYNC_FOUND_TAG,
+  ASYNC_EOS
 };
 
 typedef struct _BVWSignal BVWSignal;
@@ -170,7 +171,8 @@ struct _BVWSignal
     struct
     {
       GstElement *element;
-      char *error;
+      GError *error;
+      char *debug_message;
     } error;
     struct
     {
@@ -475,8 +477,9 @@ bacon_video_widget_class_init (BaconVideoWidgetClass * klass)
 		  G_SIGNAL_RUN_LAST,
 		  G_STRUCT_OFFSET (BaconVideoWidgetClass, tick),
 		  NULL, NULL,
-		  baconvideowidget_marshal_VOID__INT64_INT64_FLOAT,
-		  G_TYPE_NONE, 3, G_TYPE_INT64, G_TYPE_INT64, G_TYPE_FLOAT);
+		  baconvideowidget_marshal_VOID__INT64_INT64_FLOAT_BOOLEAN,
+		  G_TYPE_NONE, 4, G_TYPE_INT64, G_TYPE_INT64, G_TYPE_FLOAT,
+                  G_TYPE_BOOLEAN);
 
   bvw_table_signals[BUFFERING] =
     g_signal_new ("buffering",
@@ -499,9 +502,6 @@ bacon_video_widget_class_init (BaconVideoWidgetClass * klass)
 static void
 bacon_video_widget_instance_init (BaconVideoWidget * bvw)
 {
-  int argc = 1;
-  char **argv = NULL;
-
   g_return_if_fail (bvw != NULL);
   g_return_if_fail (BACON_IS_VIDEO_WIDGET (bvw));
 
@@ -509,14 +509,6 @@ bacon_video_widget_instance_init (BaconVideoWidget * bvw)
   GTK_WIDGET_SET_FLAGS (GTK_WIDGET (bvw), GTK_NO_WINDOW);
   GTK_WIDGET_UNSET_FLAGS (GTK_WIDGET (bvw), GTK_DOUBLE_BUFFERED);
 
-  /* we could actually change this to have some more flags if some
-   * gconf variable was set
-   * FIXME */
-  gst_init (&argc, &argv);
-  
-  /* Using opt as default scheduler */
-  gst_scheduler_factory_set_default_name ("opt");
-
   bvw->priv = g_new0 (BaconVideoWidgetPrivate, 1);
   
   bvw->priv->queue = g_async_queue_new ();
@@ -573,7 +565,7 @@ bacon_video_widget_signal_idler (BaconVideoWidget *bvw)
   
   switch (signal->signal_id)
     {
-      case VIDEO_SIZE:
+      case ASYNC_VIDEO_SIZE:
         {
           gint width, height;
           width = signal->signal_data.video_size.width;
@@ -592,13 +584,13 @@ bacon_video_widget_signal_idler (BaconVideoWidget *bvw)
           }
           break;
         }
-      case GST_ERROR:
+      case ASYNC_ERROR:
         {
           char *error_message = NULL;
           gboolean emit = TRUE;
           
           if (signal->signal_data.error.error)
-            error_message = signal->signal_data.error.error;
+            error_message = signal->signal_data.error.error->message;
 
           if (bvw->priv->last_error_message) {
             /* Let's check the latest error message */
@@ -610,13 +602,22 @@ bacon_video_widget_signal_idler (BaconVideoWidget *bvw)
           if (emit) {
             g_signal_emit (G_OBJECT (bvw),
                            bvw_table_signals[ERROR], 0, error_message, TRUE);
+            
+            /* Keep a copy of the last emitted message */
             if (bvw->priv->last_error_message)
               g_free (bvw->priv->last_error_message);
             bvw->priv->last_error_message = g_strdup (error_message);
           }
+          
+          /* Cleaning the error infos */
+          if (signal->signal_data.error.error)
+            g_error_free (signal->signal_data.error.error);
+          if (signal->signal_data.error.debug_message)
+            g_free (signal->signal_data.error.debug_message);
+          
           break;
         }
-      case FOUND_TAG:
+      case ASYNC_FOUND_TAG:
         {
           GstTagList *tag_list = signal->signal_data.found_tag.tag_list;
           if (GST_IS_TAG_LIST (tag_list)) {
@@ -627,6 +628,13 @@ bacon_video_widget_signal_idler (BaconVideoWidget *bvw)
           }
           break;
         }
+      case ASYNC_EOS:
+        {
+          gst_element_set_state (GST_ELEMENT (bvw->priv->play),
+                                 GST_STATE_READY);
+          g_signal_emit (G_OBJECT (bvw), bvw_table_signals[EOS], 0, NULL);
+          break;
+        }
       default:
         break;
     }
@@ -647,7 +655,7 @@ got_found_tag (GstPlay *play, GstElement *source,
   g_return_if_fail (BACON_IS_VIDEO_WIDGET (bvw));
   
   signal = g_new0 (BVWSignal, 1);
-  signal->signal_id = FOUND_TAG;
+  signal->signal_id = ASYNC_FOUND_TAG;
   signal->signal_data.found_tag.source = source;
   signal->signal_data.found_tag.tag_list = gst_tag_list_copy (tag_list);
 
@@ -666,7 +674,7 @@ got_video_size (GstPlay * play, gint width, gint height,
   g_return_if_fail (BACON_IS_VIDEO_WIDGET (bvw));
   
   signal = g_new0 (BVWSignal, 1);
-  signal->signal_id = VIDEO_SIZE;
+  signal->signal_id = ASYNC_VIDEO_SIZE;
   signal->signal_data.video_size.width = width;
   signal->signal_data.video_size.height = height;
 
@@ -678,9 +686,17 @@ got_video_size (GstPlay * play, gint width, gint height,
 static void
 got_eos (GstPlay * play, BaconVideoWidget * bvw)
 {
+  BVWSignal *signal;
+  
   g_return_if_fail (bvw != NULL);
   g_return_if_fail (BACON_IS_VIDEO_WIDGET (bvw));
-  g_signal_emit (G_OBJECT (bvw), bvw_table_signals[EOS], 0, NULL);
+  
+  signal = g_new0 (BVWSignal, 1);
+  signal->signal_id = ASYNC_EOS;
+
+  g_async_queue_push (bvw->priv->queue, signal);
+
+  g_idle_add ((GSourceFunc) bacon_video_widget_signal_idler, bvw);
 }
 
 static void
@@ -698,6 +714,8 @@ got_stream_length (GstPlay * play, gint64 length_nanos,
 static void
 got_time_tick (GstPlay * play, gint64 time_nanos, BaconVideoWidget * bvw)
 {
+  gboolean seekable;
+
   g_return_if_fail (bvw != NULL);
   g_return_if_fail (BACON_IS_VIDEO_WIDGET (bvw));
 
@@ -715,16 +733,19 @@ got_time_tick (GstPlay * play, gint64 time_nanos, BaconVideoWidget * bvw)
       bvw->priv->current_position =
         (float) bvw->priv->current_time / bvw->priv->stream_length;
     }
+  
+  seekable = bacon_video_widget_is_seekable (bvw);
 
   g_signal_emit (G_OBJECT (bvw),
                  bvw_table_signals[TICK], 0,
                  bvw->priv->current_time, bvw->priv->stream_length,
-                 bvw->priv->current_position);
+                 bvw->priv->current_position,
+                 seekable);
 }
 
 static void
-got_error (GstPlay * play, GstElement * orig,
-           char *error_message, BaconVideoWidget * bvw)
+got_error (GstPlay *play, GstElement *orig, GError *error,
+           gchar *debug, BaconVideoWidget * bvw)
 {
   BVWSignal *signal;
   
@@ -732,9 +753,11 @@ got_error (GstPlay * play, GstElement * orig,
   g_return_if_fail (BACON_IS_VIDEO_WIDGET (bvw));
   
   signal = g_new0 (BVWSignal, 1);
-  signal->signal_id = GST_ERROR;
+  signal->signal_id = ASYNC_ERROR;
   signal->signal_data.error.element = orig;
-  signal->signal_data.error.error = g_strdup (error_message);
+  signal->signal_data.error.error = g_error_copy (error);
+  if (debug)
+    signal->signal_data.error.debug_message = g_strdup (debug);
 
   g_async_queue_push (bvw->priv->queue, signal);
 
@@ -1074,13 +1097,22 @@ bacon_video_widget_dvd_event (BaconVideoWidget * bvw,
 void
 bacon_video_widget_set_logo (BaconVideoWidget * bvw, gchar * filename)
 {
+  GError *error = NULL;
+  
   g_return_if_fail (bvw != NULL);
   g_return_if_fail (BACON_IS_VIDEO_WIDGET (bvw));
   g_return_if_fail (GST_IS_VIDEO_WIDGET (bvw->priv->vw));
 
-  bvw->priv->logo_pixbuf = gdk_pixbuf_new_from_file (filename, NULL);
+  bvw->priv->logo_pixbuf = gdk_pixbuf_new_from_file (filename, &error);
 
-  gst_video_widget_set_logo (bvw->priv->vw, bvw->priv->logo_pixbuf);
+  if (error) {
+    g_warning ("An error occured trying to open logo %s: %s",
+               filename, error->message);
+    g_error_free (error);
+  }
+  else {
+    gst_video_widget_set_logo (bvw->priv->vw, bvw->priv->logo_pixbuf);
+  }
 }
 
 void
@@ -1180,6 +1212,24 @@ bacon_video_widget_get_volume (BaconVideoWidget * bvw)
   return volume;
 }
 
+gboolean
+bacon_video_widget_fullscreen_mode_available (BaconVideoWidget *bvw,
+		TvOutType tvout) 
+{
+	switch(tvout) {
+	case TV_OUT_NONE:
+		/* Asume that ordinary fullscreen always works */
+		return TRUE;
+	case TV_OUT_NVTV_NTSC:
+	case TV_OUT_NVTV_PAL:
+		return FALSE;
+	case TV_OUT_DXR3:
+		/* FIXME: Add DXR3 detection code */
+		return FALSE;
+	}
+	return FALSE;
+}
+
 void
 bacon_video_widget_set_fullscreen (BaconVideoWidget * bvw,
 				   gboolean fullscreen)
@@ -1634,7 +1684,7 @@ bacon_video_widget_get_mrls (BaconVideoWidget * bvw, MediaType type)
     {
       case MEDIA_DVD:
         {
-          mrls = g_malloc (sizeof (char *) * 1);
+          mrls = g_malloc0 (sizeof (char *) * 2);
           mrls[0] = "dvd://";
           return mrls;
         }
@@ -1642,7 +1692,7 @@ bacon_video_widget_get_mrls (BaconVideoWidget * bvw, MediaType type)
         return NULL;
       case MEDIA_CDDA:
         {
-          mrls = g_malloc (sizeof (char *) * 1);
+          mrls = g_malloc0 (sizeof (char *) * 2);
           mrls[0] = "cda://";
           return mrls;
         }
@@ -1853,6 +1903,14 @@ bacon_video_widget_get_type (void)
   return bacon_video_widget_type;
 }
 
+struct poptOption *
+bacon_video_widget_get_popt_table (void)
+{
+  /* Initializing GStreamer backend and parse our options from the command 
+     line options */
+  return (struct poptOption *) gst_init_get_popt_table ();
+}
+
 GtkWidget *
 bacon_video_widget_new (int width, int height,
 			gboolean null_out, GError ** err)
@@ -1861,14 +1919,20 @@ bacon_video_widget_new (int width, int height,
   GstElement *audio_sink = NULL, *video_sink = NULL;
 
   bvw = BACON_VIDEO_WIDGET (g_object_new
-			    (bacon_video_widget_get_type (), NULL));
-
+                            (bacon_video_widget_get_type (), NULL));
+  
+  bvw->priv->play = gst_play_new (err);
+  
+  if (*err != NULL) {
+    return NULL;
+  }
+  
   bvw->priv->media_device = g_strdup ("/dev/dvd");
   
   bvw->priv->metadata_hash =
     g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
 
-  bvw->priv->play = gst_play_new (NULL);
+  
 
   bvw->priv->init_width = bvw->priv->init_height = 0;
 
