@@ -16,13 +16,18 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: bacon-video-widget-xine.c,v 1.94 2004/01/31 20:47:11 trow Exp $
+ * $Id: bacon-video-widget-xine.c,v 1.108 2004/04/08 11:10:24 hadess Exp $
  *
  * the xine engine in a widget - implementation
  */
 
 #include <config.h>
 
+#ifdef HAVE_NVTV
+#include <nvtv_simple.h>
+#endif 
+
+
 /* system */
 #include <math.h>
 #include <string.h>
@@ -198,8 +203,6 @@ static gboolean bacon_video_widget_motion_notify (GtkWidget *widget,
 		GdkEventMotion *event);
 static gboolean bacon_video_widget_button_press (GtkWidget *widget,
 		GdkEventButton *event);
-static gboolean bacon_video_widget_key_press (GtkWidget *widget,
-		GdkEventKey *event);
 
 static void bacon_video_widget_size_request (GtkWidget *widget,
 		GtkRequisition *requisition);
@@ -266,7 +269,6 @@ bacon_video_widget_class_init (BaconVideoWidgetClass *klass)
 	widget_class->expose_event = bacon_video_widget_expose;
 	widget_class->motion_notify_event = bacon_video_widget_motion_notify;
 	widget_class->button_press_event = bacon_video_widget_button_press;
-	widget_class->key_press_event = bacon_video_widget_key_press;
 
 	/* GObject */
 	object_class->set_property = bacon_video_widget_set_property;
@@ -349,9 +351,9 @@ bacon_video_widget_class_init (BaconVideoWidgetClass *klass)
 				G_SIGNAL_RUN_LAST,
 				G_STRUCT_OFFSET (BaconVideoWidgetClass, tick),
 				NULL, NULL,
-				baconvideowidget_marshal_VOID__INT64_INT64_FLOAT,
-				G_TYPE_NONE, 3, G_TYPE_INT64, G_TYPE_INT64,
-				G_TYPE_FLOAT);
+				baconvideowidget_marshal_VOID__INT64_INT64_FLOAT_BOOLEAN,
+				G_TYPE_NONE, 4, G_TYPE_INT64, G_TYPE_INT64,
+				G_TYPE_FLOAT, G_TYPE_BOOLEAN);
 
 	bvw_table_signals[BUFFERING] =
 		g_signal_new ("buffering",
@@ -382,6 +384,8 @@ bacon_video_widget_instance_init (BaconVideoWidget *bvw)
 	g_return_if_fail (BACON_IS_VIDEO_WIDGET (bvw));
 
 	GTK_WIDGET_SET_FLAGS (GTK_WIDGET (bvw), GTK_CAN_FOCUS);
+	/* We work around white artifacts by using a double-buffered widget */
+	/* FIXME, this work-around breaks exposes */
 	GTK_WIDGET_UNSET_FLAGS (GTK_WIDGET (bvw), GTK_DOUBLE_BUFFERED);
 
 	bvw->priv = g_new0 (BaconVideoWidgetPrivate, 1);
@@ -418,6 +422,8 @@ bacon_video_widget_instance_init (BaconVideoWidget *bvw)
 	{
 		if (g_ascii_strcasecmp (autoplug_list[i], "VCD") == 0)
 			bvw->priv->can_vcd = TRUE;
+		else if (g_ascii_strcasecmp (autoplug_list[i], "VCDO") == 0)
+			bvw->priv->can_vcd = TRUE;
 		else if (g_ascii_strcasecmp (autoplug_list[i], "DVD") == 0)
 			bvw->priv->can_dvd = TRUE;
 		else if (g_ascii_strcasecmp (autoplug_list[i], "CD") == 0)
@@ -434,10 +440,15 @@ bacon_video_widget_finalize (GObject *object)
 {
 	BaconVideoWidget *bvw = (BaconVideoWidget *) object;
 
+	if (bvw->priv->xine != NULL) {
+		xine_exit (bvw->priv->xine);
+	}
+	g_free (bvw->priv->vis_name);
+	g_object_unref (G_OBJECT (bvw->priv->gc));
+
 	g_list_foreach (bvw->priv->visuals, (GFunc) g_free, NULL);
 	g_list_free (bvw->priv->visuals);
 
-	/* Should put here what needs to be destroyed */
 	g_idle_remove_by_data (bvw);
 	g_async_queue_unref (bvw->priv->queue);
 	G_OBJECT_CLASS (parent_class)->finalize (object);
@@ -566,6 +577,14 @@ load_video_out_driver (BaconVideoWidget *bvw, gboolean null_out)
 			"video.driver", "auto", "video driver to use",
 			NULL, 10, NULL, NULL);
 
+	/* Don't try to load anything but the xshm plugin if we're not
+	 * on a local display */
+	if (totem_display_is_local () == FALSE)
+	{
+		return xine_open_video_driver (bvw->priv->xine, "xshm",
+				XINE_VISUAL_TYPE_X11, (void *) &vis); 
+	}
+
 	if (strcmp (video_driver_id, "auto") != 0)
 	{
 		vo_driver = xine_open_video_driver (bvw->priv->xine,
@@ -670,6 +689,35 @@ bvw_config_help_num (xine_t *xine, const char *id, int val,
 	}
 }
 
+static void
+setup_url_handlers (BaconVideoWidget *bvw, const char *protocol)
+{
+	char *key, *url;
+
+	key = g_strdup_printf ("/desktop/gnome/url-handlers/%s/command",
+			protocol);
+	url = gconf_client_get_string (bvw->priv->gc, key, NULL);
+
+	if (url == NULL)
+	{
+		gconf_client_set_string (bvw->priv->gc, key,
+				"totem \"%s\"", NULL);
+		g_free (key);
+
+		key = g_strdup_printf
+			("/desktop/gnome/url-handlers/%s/need-terminal",
+			 protocol);
+		gconf_client_set_bool (bvw->priv->gc, key, FALSE, NULL);
+		g_free (key);
+
+		key =  g_strdup_printf
+			("/desktop/gnome/url-handlers/%s/enabled", protocol);
+		gconf_client_set_bool (bvw->priv->gc, key, TRUE, NULL);
+		g_free (key);
+	}
+
+	g_free (url);
+}
 
 static void
 setup_config (BaconVideoWidget *bvw)
@@ -704,10 +752,15 @@ setup_config (BaconVideoWidget *bvw)
 			NULL, 10, NULL, NULL);
 	bvw->priv->volume = -1;
 
-	/* Proxy configuration */
 	if (bvw->priv->gc == NULL)
 		return;
 
+	/* Setup the protocol handlers for our funky stuff */
+	setup_url_handlers (bvw, "pnm");
+	setup_url_handlers (bvw, "mms");
+	setup_url_handlers (bvw, "rtsp");
+
+	/* Proxy configuration */
 	if (gconf_client_get_bool (bvw->priv->gc, "/system/http_proxy/use_http_proxy", NULL) == FALSE)
 	{
 		bvw_config_helper_string (bvw->priv->xine,
@@ -771,6 +824,11 @@ setup_config_video (BaconVideoWidget *bvw)
 	bvw_config_help_num (bvw->priv->xine, "video.xv_colorkey", 30, &entry);
 	entry.num_value = 30;
 	xine_config_update_entry (bvw->priv->xine, &entry);
+
+	/* Remove the ALSA HW mixing */
+	bvw_config_help_num (bvw->priv->xine, "audio.alsa_hw_mixer", 0, &entry);
+	entry.num_value = 0;
+	xine_config_update_entry (bvw->priv->xine, &entry);
 }
 
 static void
@@ -784,11 +842,25 @@ setup_config_stream (BaconVideoWidget *bvw)
 	/* Setup brightness and contrast */
 	for (i = 0; i < 4; i++)
 	{
-		value = gconf_client_get_int (bvw->priv->gc, video_props_str[i], NULL);
+		GError *error = NULL;
+		int tmp;
+
+		value = gconf_client_get_int (bvw->priv->gc,
+				video_props_str[i], &error);
+
 		/* avoid black screens */
-		if (value == 0)
+		if (value == 0 && error != NULL)
 			value = 65535 / 2;
-		xine_set_param (bvw->priv->stream, video_props[i], value);
+
+		tmp = xine_get_param (bvw->priv->stream, video_props[i]);
+		if (value != tmp)
+		{
+			xine_set_param (bvw->priv->stream,
+					video_props[i], value);
+		}
+
+		if (error != NULL)
+			g_error_free (error);
 	}
 }
 
@@ -913,6 +985,10 @@ generate_mouse_event (BaconVideoWidget *bvw, GdkEvent *event,
 	int x, y;
 	gboolean retval;
 
+	/* Don't even try to generate an event if we're dying */
+	if (bvw->priv->stream == NULL)
+		return FALSE;
+
 	if (is_motion == FALSE && bevent->button != 1)
 		return FALSE;
 
@@ -998,7 +1074,12 @@ bacon_video_widget_realize (GtkWidget *widget)
 	xine_close (bvw->priv->stream);
 	xine_event_dispose_queue (bvw->priv->ev_queue);
 	xine_dispose (bvw->priv->stream);
-	xine_close_video_driver(bvw->priv->xine, bvw->priv->vo_driver);
+	bvw->priv->stream = NULL;
+
+	if (bvw->priv->vo_driver != NULL)
+		xine_close_video_driver (bvw->priv->xine, bvw->priv->vo_driver);
+	if (bvw->priv->ao_driver != NULL)
+		xine_close_audio_driver (bvw->priv->xine, bvw->priv->ao_driver);
 
 	/* Create the widget's window */
 	attr.x = widget->allocation.x;
@@ -1038,8 +1119,19 @@ bacon_video_widget_realize (GtkWidget *widget)
 			(gdk_display_get_default ()));
 	bvw->priv->screen = DefaultScreen (bvw->priv->display);
 
+	bvw->priv->ao_driver = load_audio_out_driver (bvw, NULL);
 	bvw->priv->vo_driver = load_video_out_driver (bvw, bvw->priv->null_out);
+
+	if (bvw->priv->vo_driver == NULL)
+	{
+		bvw->priv->vo_driver = load_video_out_driver (bvw, TRUE);
+		g_signal_emit (G_OBJECT (bvw),
+				bvw_table_signals[ERROR], 0,
+				_("No video output is available. Make sure that the program is correctly installed."), TRUE);
+		g_message ("error");
+	}
 	//FIXME
+
 	g_assert (bvw->priv->vo_driver != NULL);
 	setup_config_video (bvw);
 
@@ -1060,6 +1152,12 @@ bacon_video_widget_realize (GtkWidget *widget)
 	xine_event_create_listener_thread (bvw->priv->ev_queue,
 			xine_event, (void *) bvw);
 
+#ifdef HAVE_NVTV
+	if (!(nvtv_simple_init() && nvtv_enable_autoresize(TRUE))) {
+		nvtv_simple_enable(FALSE);
+	} 
+#endif
+
 	return;
 }
 
@@ -1129,13 +1227,6 @@ xine_event_message (BaconVideoWidget *bvw, xine_ui_message_data_t *data)
 
 	message = NULL;
 
-	if (!(data->explanation))
-	{
-		D("xine_event_message: UI message without an explanation\n"
-				"type: %d", data->type);
-		return;
-	}
-
 	switch(data->type)
 	{
 	case XINE_MSG_NO_ERROR:
@@ -1172,12 +1263,15 @@ xine_event_message (BaconVideoWidget *bvw, xine_ui_message_data_t *data)
 	case XINE_MSG_SECURITY:
 		message = g_strdup (_("For security reasons, this movie can not be played back."));
 		break;
+	case XINE_MSG_AUDIO_OUT_UNAVAILABLE:
+		xine_stop (bvw->priv->stream);
+		message = g_strdup (_("The audio device is busy. Is another application using it?"));
+		break;
 	}
 
 	if (message == NULL)
 	{
-		D("xine_event_message: unhandled error\n"
-				"type: %d", data->type);
+		D("xine_event_message: unhandled error\ntype: %d", data->type);
 		return;
 	}
 
@@ -1244,6 +1338,9 @@ xine_event (void *user_data, const xine_event_t *event)
 		g_async_queue_push (bvw->priv->queue, data);
 		g_idle_add ((GSourceFunc) bacon_video_widget_idle_signal, bvw);
 		break;
+	case XINE_EVENT_AUDIO_LEVEL:
+		/* Unhandled, we use the software mixer, not the hardware one */
+		break;
 	}
 }
 
@@ -1303,8 +1400,19 @@ bacon_video_widget_unrealize (GtkWidget *widget)
 	xine_stop (bvw->priv->stream);
 	xine_close (bvw->priv->stream);
 
+	/* Put the current volume in the config system */
+	if (bacon_video_widget_can_set_volume (bvw) != FALSE)
+	{
+		xine_config_lookup_entry (bvw->priv->xine,
+				"misc.amp_level", &entry);
+		entry.num_value = bvw->priv->volume;
+		xine_config_update_entry (bvw->priv->xine, &entry);
+	}
+
 	/* Kill the TV out */
-	xine_tvmode_exit (bvw->priv->xine);
+#ifdef HAVE_NVTV
+        nvtv_simple_exit();
+#endif
 
 	/* Hide all windows */
 	if (GTK_WIDGET_MAPPED (widget))
@@ -1312,15 +1420,10 @@ bacon_video_widget_unrealize (GtkWidget *widget)
 	GTK_WIDGET_UNSET_FLAGS (widget, GTK_MAPPED);
 
 	/* Get rid of the rest of the stream */
-	xine_event_dispose_queue (bvw->priv->ev_queue);
 	xine_dispose (bvw->priv->stream);
+	xine_event_dispose_queue (bvw->priv->ev_queue);
 	bvw->priv->stream = NULL;
 
-	/* Put the current volume in the config system */
-	xine_config_lookup_entry (bvw->priv->xine, "misc.amp_level", &entry);
-	entry.num_value = bvw->priv->volume;
-	xine_config_update_entry (bvw->priv->xine, &entry);
-
 	/* save config */
 	configfile = g_build_path (G_DIR_SEPARATOR_S,
 			g_get_home_dir (), CONFIG_FILE, NULL);
@@ -1344,6 +1447,17 @@ bacon_video_widget_unrealize (GtkWidget *widget)
 		(*GTK_WIDGET_CLASS (parent_class)->unrealize) (widget);
 }
 
+static struct poptOption xine_options[] = {
+	POPT_TABLEEND
+};
+
+struct poptOption *
+bacon_video_widget_get_popt_table (void)
+{
+	/* Xine backend does not need any options */
+	return (struct poptOption *) xine_options;
+}
+
 GtkWidget *
 bacon_video_widget_new (int width, int height, gboolean null_out, GError **err)
 {
@@ -1375,14 +1489,38 @@ bacon_video_widget_new (int width, int height, gboolean null_out, GError **err)
 	bvw->priv->init_width = width;
 	bvw->priv->init_height = height;
 
-	/* load the video drivers */
+	/* load the output drivers */
 	bvw->priv->ao_driver = load_audio_out_driver (bvw, err);
 	if (err != NULL && *err != NULL)
 		return NULL;
 
 	bvw->priv->vo_driver = load_video_out_driver (bvw, TRUE);
-	//FIXME
-	g_assert (bvw->priv->vo_driver != NULL);
+
+	/* Be extra careful about exiting out nicely when a video output
+	 * isn't available */
+	if (bvw->priv->vo_driver == NULL)
+	{
+		/* Close the xine stuff */
+		if (bvw->priv->ao_driver != NULL) {
+			xine_close_audio_driver (bvw->priv->xine,
+					bvw->priv->ao_driver);
+		}
+		xine_exit (bvw->priv->xine);
+
+		/* get rid of all our crappety crap */
+		g_source_remove (bvw->priv->tick_id);
+		g_idle_remove_by_data (bvw);
+		g_async_queue_unref (bvw->priv->queue);
+		g_free (bvw->priv->vis_name);
+		g_object_unref (G_OBJECT (bvw->priv->gc));
+		g_free (bvw->priv->vis_name);
+		g_free (bvw->priv);
+		g_free (bvw);
+
+		g_set_error (err, 0, 0,
+				_("No video output is available. Make sure that the program is correctly installed."));
+		return NULL;
+	}
 
 	if (bvw->priv->null_out != FALSE)
 	{
@@ -1412,13 +1550,6 @@ bacon_video_widget_expose (GtkWidget *widget, GdkEventExpose *event)
 	if (event->count != 0)
 		return FALSE;
 
-	if (bvw->priv->started == FALSE)
-	{
-		gdk_draw_rectangle (widget->window, widget->style->black_gc,
-				TRUE, event->area.x, event->area.y,
-				event->area.width, event->area.height);
-	}
-
 	expose = g_new0 (XExposeEvent, 1);
 	expose->count = event->count;
 
@@ -1457,16 +1588,6 @@ bacon_video_widget_button_press (GtkWidget *widget, GdkEventButton *event)
 	return FALSE;
 }
 
-static gboolean
-bacon_video_widget_key_press (GtkWidget *widget, GdkEventKey *event)
-{
-	if (GTK_WIDGET_CLASS (parent_class)->key_press_event != NULL)
-		(* GTK_WIDGET_CLASS (parent_class)->key_press_event) (widget, event);
-
-	return FALSE;
-}
-
-
 static void
 bacon_video_widget_size_request (GtkWidget *widget, GtkRequisition *requisition)
 {
@@ -1522,7 +1643,7 @@ bacon_video_widget_tick_send (BaconVideoWidget *bvw)
 {
 	int current_time, stream_length, current_position;
 	float current_position_f;
-	gboolean ret = TRUE;
+	gboolean ret = TRUE, seekable;
 
 	if (bvw->priv->stream == NULL || bvw->priv->logo_mode != FALSE)
 		return TRUE;
@@ -1539,6 +1660,9 @@ bacon_video_widget_tick_send (BaconVideoWidget *bvw)
 				&stream_length);
 	}
 
+	if (ret == FALSE)
+		return TRUE;
+
 	if (bvw->priv->seeking == 1)
 	{
 		current_position_f = bvw->priv->seek_dest;
@@ -1555,12 +1679,19 @@ bacon_video_widget_tick_send (BaconVideoWidget *bvw)
 	else
 		bvw->priv->is_live = TRUE;
 
-	if (ret != FALSE)
-		g_signal_emit (G_OBJECT (bvw),
-				bvw_table_signals[TICK], 0,
-				(gint64) (current_time),
-				(gint64) (stream_length),
-				current_position_f);
+	if (stream_length != 0 && bvw->priv->mrl != NULL) {
+		seekable = xine_get_stream_info (bvw->priv->stream,
+				XINE_STREAM_INFO_SEEKABLE);
+	} else {
+		seekable = FALSE;
+	}
+
+	g_signal_emit (G_OBJECT (bvw),
+			bvw_table_signals[TICK], 0,
+			(gint64) (current_time),
+			(gint64) (stream_length),
+			current_position_f,
+			seekable);
 
 	return TRUE;
 }
@@ -1643,7 +1774,6 @@ bacon_video_widget_open (BaconVideoWidget *bvw, const char *mrl,
 	g_return_val_if_fail (bvw->priv->xine != NULL, FALSE);
 	g_return_val_if_fail (bvw->priv->mrl == NULL, FALSE);
 
-	bvw->priv->mrl = g_strdup (mrl);
 	bvw->priv->video_fcc = 0;
 	bvw->priv->audio_fcc = 0;
 
@@ -1672,7 +1802,7 @@ bacon_video_widget_open (BaconVideoWidget *bvw, const char *mrl,
 		fourcc_str = get_fourcc_string (bvw->priv->video_fcc);
 		name = g_strdup (xine_get_meta_info (bvw->priv->stream,
 				XINE_META_INFO_VIDEOCODEC));
-#ifdef HAVE_X86
+
 		/* Only change the audio_fcc if we're on x86 */
 		if (xine_get_stream_info
 				(bvw->priv->stream,
@@ -1682,10 +1812,6 @@ bacon_video_widget_open (BaconVideoWidget *bvw, const char *mrl,
 				(bvw->priv->stream,
 				 XINE_STREAM_INFO_AUDIO_FOURCC);
 		}
-#else
-		/* Reset video_fcc, and leave audio_fcc at 0 */
-		bvw->priv->video_fcc = 0;
-#endif
 
 		bacon_video_widget_close (bvw);
 
@@ -1716,6 +1842,8 @@ bacon_video_widget_open (BaconVideoWidget *bvw, const char *mrl,
 
 	show_vfx_update (bvw, bvw->priv->show_vfx);
 
+	bvw->priv->mrl = g_strdup (mrl);
+
 	g_signal_emit (G_OBJECT (bvw),
 			bvw_table_signals[GOT_METADATA], 0, NULL);
 
@@ -1772,6 +1900,20 @@ bacon_video_widget_play (BaconVideoWidget *bvw, GError **gerror)
 		bvw->priv->queued_vis = NULL;
 	}
 
+	/* Workaround for xine-lib: don't try to use a
+	 * non-existent audio channel */
+	{
+		int cur, num;
+
+		cur = xine_get_param(bvw->priv->stream,
+				XINE_PARAM_AUDIO_CHANNEL_LOGICAL);
+		num = xine_get_stream_info(bvw->priv->stream,
+				XINE_STREAM_INFO_AUDIO_CHANNELS);
+		if (cur > num)
+			xine_set_param(bvw->priv->stream,
+					XINE_PARAM_AUDIO_CHANNEL_LOGICAL, -1);
+	}
+
 	return TRUE;
 }
 
@@ -1836,7 +1978,6 @@ gboolean bacon_video_widget_seek_time (BaconVideoWidget *bvw, gint64 time,
 	return TRUE;
 }
 
-
 void
 bacon_video_widget_stop (BaconVideoWidget *bvw)
 {
@@ -1844,9 +1985,6 @@ bacon_video_widget_stop (BaconVideoWidget *bvw)
 	g_return_if_fail (BACON_IS_VIDEO_WIDGET (bvw));
 	g_return_if_fail (bvw->priv->xine != NULL);
 
-	if (bacon_video_widget_is_playing (bvw) == FALSE)
-		return;
-
 	xine_stop (bvw->priv->stream);
 }
 
@@ -2091,21 +2229,77 @@ bacon_video_widget_get_volume (BaconVideoWidget *bvw)
 	return bvw->priv->volume;
 }
 
+gboolean
+bacon_video_widget_fullscreen_mode_available (BaconVideoWidget *bvw,
+		TvOutType tvout) 
+{
+	switch(tvout) {
+	case TV_OUT_NONE:
+		/* Assume that ordinary fullscreen always works */
+		return TRUE;
+	case TV_OUT_NVTV_NTSC:
+	case TV_OUT_NVTV_PAL:
+#ifdef HAVE_NVTV
+		/* Make sure nvtv is initialized, it will not do any harm 
+		 * if it is done twice any way */
+		if (!(nvtv_simple_init() && nvtv_enable_autoresize(TRUE))) {
+			nvtv_simple_enable(FALSE);
+		}
+		return (nvtv_simple_is_available());
+#else
+		return FALSE;
+#endif
+	case TV_OUT_DXR3:
+		{
+			const char * const *list;
+			int i;
+
+			list = xine_list_video_output_plugins (bvw->priv->xine);
+			for (i = 0; list[i] != NULL; i++) {
+				if (strcmp ("dxr3", list[i]) == 0)
+					return TRUE;
+			}
+			return FALSE;
+		}
+	default:
+		g_assert_not_reached ();
+	}
+
+	return FALSE;
+}
+
 void
 bacon_video_widget_set_fullscreen (BaconVideoWidget *bvw, gboolean fullscreen)
 {
-	if (bvw->priv->auto_resize == FALSE)
+	if (((bvw->priv->auto_resize == FALSE) || (bvw->priv->have_xrandr == FALSE)) &&
+			(bvw->priv->tvout != TV_OUT_NVTV_NTSC) &&
+			(bvw->priv->tvout != TV_OUT_NVTV_PAL))
 		return;
 
 	bvw->priv->fullscreen_mode = fullscreen;
 
-	if (bvw->priv->have_xrandr == FALSE)
-		return;
-
 	if (fullscreen == FALSE)
 	{
+#ifdef HAVE_NVTV
+        /* If NVTV is used */
+        if (nvtv_simple_get_state() == NVTV_SIMPLE_TV_ON) {
+		nvtv_simple_switch(NVTV_SIMPLE_TV_OFF,0,0);
+        
+		/* Else if just auto resize is used */
+        } else if (bvw->priv->auto_resize != FALSE) {
+#endif
 		bacon_restore (bvw->priv->screenid);
-	} else {
+#ifdef HAVE_NVTV
+	}
+	/* Turn fullscreen on with NVTV if that option is on */
+	} else if ((bvw->priv->tvout == TV_OUT_NVTV_NTSC) ||
+			(bvw->priv->tvout == TV_OUT_NVTV_PAL)) {
+		nvtv_simple_switch(NVTV_SIMPLE_TV_ON,
+				bvw->priv->video_width,
+				bvw->priv->video_height);
+#endif
+		/* Turn fullscreen on with autoresize */
+	} else if (bvw->priv->auto_resize != FALSE) {
 		bvw->priv->screenid = bacon_resize_get_current ();
 		bacon_resize (bvw->priv->video_height,
 				bvw->priv->video_width);
@@ -2113,7 +2307,8 @@ bacon_video_widget_set_fullscreen (BaconVideoWidget *bvw, gboolean fullscreen)
 }
 
 void
-bacon_video_widget_set_show_cursor (BaconVideoWidget *bvw, gboolean show_cursor)
+bacon_video_widget_set_show_cursor (BaconVideoWidget *bvw,
+		gboolean show_cursor)
 {
 	g_return_if_fail (bvw != NULL);
 	g_return_if_fail (BACON_IS_VIDEO_WIDGET (bvw));
@@ -2282,11 +2477,13 @@ bacon_video_widget_set_tv_out (BaconVideoWidget *bvw, TvOutType tvout)
 		return TRUE;
 	}
 
-	if (tvout == TV_OUT_TVMODE || bvw->priv->tvout == TV_OUT_TVMODE)
-	{
-		xine_tvmode_switch (bvw->priv->xine,
-				(tvout == TV_OUT_TVMODE), 0, 0, 0);
+#ifdef HAVE_NVTV
+	if (tvout == TV_OUT_NVTV_PAL) {
+		nvtv_simple_set_tvsystem(NVTV_SIMPLE_TVSYSTEM_PAL);
+	} else if (tvout == TV_OUT_NVTV_NTSC) {
+		nvtv_simple_set_tvsystem(NVTV_SIMPLE_TVSYSTEM_NTSC);
 	}
+#endif
 
 	bvw->priv->tvout = tvout;
 
@@ -2556,6 +2753,9 @@ bacon_video_widget_is_seekable (BaconVideoWidget *bvw)
 	g_return_val_if_fail (BACON_IS_VIDEO_WIDGET (bvw), 0);
 	g_return_val_if_fail (bvw->priv->xine != NULL, 0);
 
+	if (bvw->priv->mrl == NULL)
+		return FALSE;
+
 	if (bacon_video_widget_get_stream_length (bvw) == 0)
 		return FALSE;
 
@@ -3089,7 +3289,8 @@ bacon_video_widget_can_get_frames (BaconVideoWidget *bvw, GError **error)
 	g_return_val_if_fail (BACON_IS_VIDEO_WIDGET (bvw), FALSE);
 	g_return_val_if_fail (bvw->priv->xine != NULL, FALSE);
 
-	if (bacon_video_widget_is_playing (bvw) == FALSE)
+	if (xine_get_status (bvw->priv->stream) != XINE_STATUS_PLAY
+			&& bvw->priv->logo_mode == FALSE)
 	{
 		g_set_error (error, 0, 0, _("Movie is not playing"));
 		return FALSE;
