@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: bacon-video-widget.h,v 1.37 2004/01/15 21:46:36 dolphy Exp $
+ * $Id: bacon-video-widget.h,v 1.40 2004/03/31 16:23:32 hadess Exp $
  *
  * the xine engine in a widget - header
  */
@@ -25,6 +25,7 @@
 #define HAVE_BACON_VIDEO_WIDGET_H
 
 #include <gtk/gtkbox.h>
+#include <popt.h>
 
 G_BEGIN_DECLS
 
@@ -50,12 +51,13 @@ typedef struct {
 	void (*title_change) (GtkWidget *bvw, const char *title);
 	void (*channels_change) (GtkWidget *bvw);
 	void (*tick) (GtkWidget *bvw, gint64 current_time, gint64 stream_length,
-			float current_position);
+			float current_position, gboolean seekable);
 	void (*buffering) (GtkWidget *bvw, guint progress);
 	void (*speed_warning) (GtkWidget *bvw);
 } BaconVideoWidgetClass;
 
 GType bacon_video_widget_get_type                (void);
+struct poptOption *bacon_video_widget_get_popt_table    (void);
 GtkWidget *bacon_video_widget_new		 (int width, int height,
 						  gboolean null_video_out,
 						  GError **error);
@@ -237,9 +239,13 @@ GdkPixbuf *bacon_video_widget_get_current_frame (BaconVideoWidget *bvw);
 typedef enum {
 	TV_OUT_NONE,
 	TV_OUT_DXR3,
-	TV_OUT_TVMODE
+	TV_OUT_NVTV_PAL,
+	TV_OUT_NVTV_NTSC
 } TvOutType;
 
+gboolean bacon_video_widget_fullscreen_mode_available (BaconVideoWidget *bvw,
+						       TvOutType tvout);
+
 gboolean bacon_video_widget_set_tv_out           (BaconVideoWidget *bvw,
 						  TvOutType tvout);
 TvOutType bacon_video_widget_get_tv_out          (BaconVideoWidget *bvw);
