@@ -23,8 +23,11 @@
    Bastien Nocera <hadess@hadess.net>
 */
 
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif
 
+#include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <sys/types.h>
@@ -35,6 +38,10 @@
 #include <fcntl.h>
 #include <math.h>
 
+#ifdef USE_HAL
+#include <libhal.h>
+#endif /* USE_HAL */
+
 #ifdef __linux__
 #include <scsi/scsi.h>
 #include <scsi/sg.h>
@@ -47,7 +54,19 @@
 #endif /* __FreeBSD__ */
 
 #include <glib.h>
-#include <libgnome/gnome-i18n.h>
+
+#ifdef ENABLE_NLS
+#include <libintl.h>
+#define _(String) dgettext(GETTEXT_PACKAGE,String)
+#ifdef gettext_noop
+#define N_(String) gettext_noop(String)
+#else
+#define N_(String) (String)
+#endif /* gettext_noop */
+#else
+#define _(String) (String)
+#define N_(String) (String)
+#endif /* ENABLE_NLS */
 
 #include "cd-drive.h"
 
@@ -61,11 +80,158 @@ static struct {
 	{ "IOMEGA - CDRW9602EXT-B", TRUE, TRUE, FALSE, FALSE },
 };
 
+#ifdef USE_HAL
+
+static void
+hal_scan_add_whitelist (CDDrive *cdrom)
+{
+	guint i;
+
+	for (i = 0; i < G_N_ELEMENTS (recorder_whitelist); i++) {
+		if (!strcmp (cdrom->display_name, recorder_whitelist[i].name)) {
+			if (recorder_whitelist[i].can_write_cdr) {
+				cdrom->type |= CDDRIVE_TYPE_CD_RECORDER;
+			}
+			if (recorder_whitelist[i].can_write_cdrw) {
+				cdrom->type |= CDDRIVE_TYPE_CDRW_RECORDER;
+			}
+			if (recorder_whitelist[i].can_write_dvdr) {
+				cdrom->type |= CDDRIVE_TYPE_DVD_RW_RECORDER;
+			}
+			if (recorder_whitelist[i].can_write_dvdram) {
+				cdrom->type |= CDDRIVE_TYPE_DVD_RAM_RECORDER;
+			}
+		}
+	}
+}
+
+#define GET_BOOL_PROP(x) (hal_device_property_exists (device_names[i], x) && hal_device_get_property_bool (device_names[i], x))
+#define CD_ROM_SPEED 176
+
+static GList *
+hal_scan (gboolean recorder_only)
+{
+	GList *cdroms = NULL;
+	LibHalFunctions hal_functions = {
+		NULL, /* mainloop integration */
+		NULL, /* device_added */
+		NULL, /* device_removed */
+		NULL, /* device_new_capability */
+		NULL, /* property_modified */
+		NULL, /* device_condition */
+	};
+	int i;
+	int num_devices;
+	char** device_names;
+
+	if (hal_initialize (&hal_functions, FALSE)) {
+		return NULL;
+	}
+
+	device_names = hal_get_all_devices (&num_devices);
+	if (device_names == NULL)
+	{
+		hal_shutdown ();
+		return NULL;
+	}
+
+	for (i = 0; i < num_devices; i++)
+	{
+		CDDrive *cdrom;
+		char *string;
+		gboolean is_cdr;
+
+		/* Is it a removable drive? */
+		if (!GET_BOOL_PROP ("storage.removable")) {
+			continue;
+		}
+
+		/* Is it a CD drive? */
+		string = hal_device_get_property_string (device_names[i],
+				"storage.media");
+		if (string == NULL || strcmp (string, "cdrom") != 0) {
+			g_free (string);
+			continue;
+		}
+		g_free (string);
+
+		/* Is it the CD-drive, or a volume on the media? */
+		if (GET_BOOL_PROP ("block.is_volume") != FALSE) {
+			continue;
+		}
+
+		/* Is it a CD burner? */
+		is_cdr = GET_BOOL_PROP ("storage.cdr");
+		if (recorder_only && is_cdr == FALSE) {
+			continue;
+		}
+
+		cdrom = g_new0 (CDDrive, 1);
+		cdrom->type = CDDRIVE_TYPE_CD_DRIVE;
+		if (is_cdr != FALSE) {
+			cdrom->type |= CDDRIVE_TYPE_CD_RECORDER;
+		}
+		if (GET_BOOL_PROP ("storage.cdrw")) {
+			cdrom->type |= CDDRIVE_TYPE_CDRW_RECORDER;
+		}
+		if (GET_BOOL_PROP ("storage.dvd")) {
+			cdrom->type |= CDDRIVE_TYPE_DVD_DRIVE;
+
+			if (GET_BOOL_PROP ("storage.dvdram")) {
+				cdrom->type |= CDDRIVE_TYPE_DVD_RAM_RECORDER;
+			}
+			if (GET_BOOL_PROP ("storage.dvdr")) {
+				cdrom->type |= CDDRIVE_TYPE_DVD_RW_RECORDER;
+			}
+			if (GET_BOOL_PROP ("storage.dvd")) {
+				cdrom->type |= CDDRIVE_TYPE_DVD_DRIVE;
+			}
+			if (GET_BOOL_PROP ("storage.dvdplusr")) {
+				cdrom->type |= CDDRIVE_TYPE_DVD_PLUS_R_RECORDER;
+			}
+			if (GET_BOOL_PROP ("storage.dvdplusrw")) {
+				cdrom->type |= CDDRIVE_TYPE_DVD_PLUS_RW_RECORDER;
+			}
+		}
+
+		cdrom->device = hal_device_get_property_string (device_names[i],
+				"block.device");
+		cdrom->cdrecord_id = g_strdup (cdrom->device);
+
+		string = hal_device_get_property_string (device_names[i],
+				"storage.model");
+		if (string != NULL) {
+			cdrom->display_name = string;
+		} else {
+			cdrom->display_name = g_strdup_printf ("Unnamed CD-ROM (%s)", cdrom->device);
+		}
+
+		cdrom->max_speed_read = hal_device_get_property_int
+			(device_names[i], "storage.cdrom.read_speed")
+			/ CD_ROM_SPEED;
+
+		if (hal_device_property_exists (device_names[i], "storage.cdrom.write_speed")) {
+			cdrom->max_speed_write = hal_device_get_property_int
+				(device_names[i], "storage.cdrom.write_speed")
+				/ CD_ROM_SPEED;
+		}
+
+		hal_scan_add_whitelist (cdrom);
+
+		cdroms = g_list_prepend (cdroms, cdrom);
+	}
+
+	cdroms = g_list_reverse (cdroms);
+
+	return cdroms;
+}
+#endif /* USE_HAL */
+
 #if defined(__linux__) || defined(__FreeBSD__)
 
 /* For dvd_plus_rw_utils.cpp */
 int get_dvd_r_rw_profile (const char *name);
-int get_mmc_profile (void *fd);
+int get_mmc_profile (int fd);
 
 static void
 add_dvd_plus (CDDrive *cdrom)
@@ -94,12 +260,15 @@ linux_bsd_media_type (const char *device)
 	int fd;
 	int mmc_profile;
 
-	fd = open (device, O_RDONLY);
+	fd = open (device, O_RDONLY|O_EXCL);
 	if (fd < 0) {
+		if (errno == EBUSY) {
+			return CD_MEDIA_TYPE_BUSY;
+		}
 		return CD_MEDIA_TYPE_ERROR;
 	}
 
-	mmc_profile = get_mmc_profile ((void *)&fd);
+	mmc_profile = get_mmc_profile (fd);
 	close (fd);
 
 	switch (mmc_profile) {
@@ -196,6 +365,7 @@ struct scsi_unit {
 };
 
 struct cdrom_unit {
+	CDProtocolType protocol;
 	char *device;
 	char *display_name;
 	int speed;
@@ -305,17 +475,22 @@ get_cd_scsi_id (const char *dev, int *bus, int *id, int *lun)
 	devfile = g_strdup_printf ("/dev/%s", dev);
 	fd = open(devfile, O_RDONLY | O_NONBLOCK);
 	g_free (devfile);
-	
+
+	/* Avoid problems with Valgrind */
+	memset (&m_idlun, 1, sizeof (m_idlun));
+	*bus = *id = *lun = -1;
+
 	if (fd < 0) {
 		g_warning ("Failed to open cd device %s\n", dev);
 		return 0;
 	}
-    
-	if (ioctl (fd, SCSI_IOCTL_GET_BUS_NUMBER, bus) < 0) {
+
+	if (ioctl (fd, SCSI_IOCTL_GET_BUS_NUMBER, bus) < 0 || *bus < 0) {
 		g_warning ("Failed to get scsi bus nr\n");
 		close (fd);
 		return 0;
 	}
+
 	if (ioctl (fd, SCSI_IOCTL_GET_IDLUN, &m_idlun) < 0) {
 		g_warning ("Failed to get scsi id and lun\n");
 		close(fd);
@@ -323,7 +498,7 @@ get_cd_scsi_id (const char *dev, int *bus, int *id, int *lun)
 	}
 	*id = m_idlun.mux4 & 0xFF;
 	*lun = (m_idlun.mux4 >> 8)  & 0xFF;
-	
+
 	close(fd);
 	return 1;
 }
@@ -361,14 +536,14 @@ get_device_max_speed (char *id)
 	argv[i++] = NULL;
 
 	if (g_spawn_sync (NULL,
-			  (char **)argv,
-			  NULL,
-			  G_SPAWN_SEARCH_PATH | G_SPAWN_STDERR_TO_DEV_NULL,
-			  NULL, NULL,
-			  &stdout_data,
-			  NULL,
-			  NULL,
-			  NULL)) {
+				(char **)argv,
+				NULL,
+				G_SPAWN_SEARCH_PATH | G_SPAWN_STDERR_TO_DEV_NULL,
+				NULL, NULL,
+				&stdout_data,
+				NULL,
+				NULL,
+				NULL)) {
 		speed = strstr (stdout_data, "Maximum write speed in kB/s:");
 		if (speed != NULL) {
 			speed += strlen ("Maximum write speed in kB/s:");
@@ -387,13 +562,12 @@ get_device_max_speed (char *id)
 	return max_speed;
 }
 
-
 static char *
 get_scsi_cd_name (int bus, int id, int lun, const char *dev,
 		  struct scsi_unit *scsi_units, int n_scsi_units)
 {
 	struct scsi_unit *scsi_unit;
-	
+
 	scsi_unit = lookup_scsi_unit (bus, id, lun, scsi_units, n_scsi_units);
 	if (scsi_unit == NULL) {
 		return g_strdup_printf (_("Unnamed SCSI CD-ROM (%s)"), dev);
@@ -420,11 +594,12 @@ cdrom_get_name (struct cdrom_unit *cdrom, struct scsi_unit *scsi_units, int n_sc
 		return NULL;
 	}
 	if (i == 2) {
+		g_free (cdrom->device);
 		cdrom->device = g_strdup(devfsname);
 	}
 	stdname[3] = '\0'; devfsname[14] = '\0'; /* just in case */
 	
-	if (stdname[0] == 's') {
+	if (cdrom->protocol == CD_PROTOCOL_SCSI) {
 		get_cd_scsi_id (cdrom->device, &bus, &id, &lun);
 		retval = get_scsi_cd_name (bus, id, lun, cdrom->device, scsi_units,
 			   n_scsi_units);
@@ -465,7 +640,8 @@ add_linux_cd_recorder (GList *cdroms,
 	cdrom->type = CDDRIVE_TYPE_CD_DRIVE;
 	cdrom->display_name = g_strdup (cdrom_s->display_name);
 
-	if (cdrom_s->device[0] == 's') {
+	if (cdrom_s->protocol == CD_PROTOCOL_SCSI) {
+		cdrom->protocol = CD_PROTOCOL_SCSI;
 		if (!get_cd_scsi_id (cdrom_s->device, &bus, &id, &lun)) {
 			g_free (cdrom->display_name);
 			g_free (cdrom);
@@ -474,6 +650,7 @@ add_linux_cd_recorder (GList *cdroms,
 		cdrom->cdrecord_id = g_strdup_printf ("%d,%d,%d",
 				bus, id, lun);
 	} else {
+		cdrom->protocol = CD_PROTOCOL_IDE;
 		/* kernel >=2.5 can write cd w/o ide-scsi */
 		cdrom->cdrecord_id = g_strdup_printf ("/dev/%s",
 				cdrom_s->device);
@@ -508,10 +685,9 @@ add_linux_cd_recorder (GList *cdroms,
 	}
 	if (cdrom_s->can_read_dvd) {
 		cdrom->type |= CDDRIVE_TYPE_DVD_DRIVE;
+		add_dvd_plus (cdrom);
 	}
 
-	add_dvd_plus (cdrom);
-
 	return g_list_append (cdroms, cdrom);
 }
 
@@ -597,6 +773,8 @@ linux_scan (gboolean recorder_only)
 			*t = 0;
 		}
 		cdroms[j].device = get_cd_device_file (p);
+		/* Assume its an IDE device for now */
+		cdroms[j].protocol = CD_PROTOCOL_IDE;
 		if (t != NULL) {
 			p = t + 1;
 		}
@@ -609,6 +787,7 @@ linux_scan (gboolean recorder_only)
 	n_scsi_units = 0;
 	for (i = 0; i < n_cdroms; i++) {
 		if (cdroms[i].device[0] == 's') {
+			cdroms[i].protocol = CD_PROTOCOL_SCSI;
 			n_scsi_units++;
 		}
 	}
@@ -616,7 +795,7 @@ linux_scan (gboolean recorder_only)
 	if (n_scsi_units > 0) {
 		/* open /dev/sg0 to force loading of the sg module if not loaded yet */
 		fd = open ("/dev/sg0", O_RDONLY);
-		if (fd != -1) {
+		if (fd >= 0) {
 			close (fd);
 		}
 		
@@ -627,7 +806,7 @@ linux_scan (gboolean recorder_only)
 			g_strfreev (devices);
 			return NULL;
 		}
-		
+
 		scsi_units = g_new0 (struct scsi_unit, n_scsi_units);
 		get_scsi_units (device_str, devices, scsi_units);
 
@@ -636,7 +815,7 @@ linux_scan (gboolean recorder_only)
 	} else {
 		scsi_units = NULL;
 	}
-	
+
 	for (i = 3; cdrom_info[i] != NULL; i++) {
 		if (g_str_has_prefix (cdrom_info[i], "Can write CD-R:")) {
 			p = cdrom_info[i] + strlen ("Can write CD-R:");
@@ -724,8 +903,11 @@ linux_scan (gboolean recorder_only)
 	cdroms_list = NULL;
 	for (i = n_cdroms - 1, j = 0; i >= 0; i--, j++) {
 		if (have_devfs) {
-			cdroms[i].device = g_strdup_printf("%s cdroms/cdrom%d",
+			char *s;
+			s = g_strdup_printf("%s cdroms/cdrom%d",
 					cdroms[i].device,  j);
+			g_free (cdroms[i].device);
+			cdroms[i].device = s;
 		}
 		cdroms[i].display_name = cdrom_get_name (&cdroms[i],
 				scsi_units, n_scsi_units);
@@ -735,7 +917,7 @@ linux_scan (gboolean recorder_only)
 		    cdroms[i].can_write_cdrw ||
 		    cdroms[i].can_write_dvdr ||
 		    cdroms[i].can_write_dvdram) &&
-			(cdroms[i].device[0] == 's' ||
+			(cdroms[i].protocol == CD_PROTOCOL_SCSI ||
 			(maj > 2) || (maj == 2 && min >= 5))) {
 			cdroms_list = add_linux_cd_recorder (cdroms_list,
 					recorder_only, &cdroms[i],
@@ -748,38 +930,149 @@ linux_scan (gboolean recorder_only)
 
 	for (i = n_cdroms - 1; i >= 0; i--) {
 		g_free (cdroms[i].display_name);
+		g_free (cdroms[i].device);
 	}
-
-	g_free (scsi_units);
 	g_free (cdroms);
 
+	for (i = n_scsi_units - 1; i >= 0; i--) {
+		g_free (scsi_units[i].vendor);
+		g_free (scsi_units[i].model);
+		g_free (scsi_units[i].rev);
+	}
+	g_free (scsi_units);
+
 	return cdroms_list;
 }
 #endif /* __linux__ */
 
 #ifdef __FreeBSD__
+static void
+get_cd_properties (char *id, int *max_rd_speed, int *max_wr_speed,
+	CDDriveType *type)
+{
+    	int i;
+	const char *argv[20];
+	char *dev_str, *stdout_data, *rd_speed, *wr_speed, *drive_cap;
+
+	*max_rd_speed = -1;
+	*max_wr_speed = -1;
+	*type = 0;
+
+	i = 0;
+	argv[i++] = "cdrecord";
+	argv[i++] = "-prcap";
+	dev_str = g_strdup_printf ("dev=%s", id);
+	argv[i++] = dev_str;
+	argv[i++] = NULL;
+
+	if (g_spawn_sync (NULL,
+			  (char **)argv,
+			  NULL,
+			  G_SPAWN_SEARCH_PATH | G_SPAWN_STDERR_TO_DEV_NULL,
+			  NULL, NULL,
+			  &stdout_data,
+			  NULL,
+			  NULL,
+			  NULL)) {
+		wr_speed = strstr (stdout_data, "Maximum write speed:");
+		if (wr_speed != NULL) {
+		    	char *tok;
+		    	wr_speed += strlen ("Maximum write speed:");
+			for (tok = strtok (wr_speed, " (),\t\n");
+				tok && strcmp (tok, "CD");
+				tok = strtok (NULL, " (),\t\n")) {}
+			tok = strtok (NULL, " (),\t\n"); /* Get the CD speed. */
+			*max_wr_speed = atol (tok);
+		}
+		else {
+		    	wr_speed = strstr (stdout_data, "Maximum write speed in kB/s:");
+			if (wr_speed != NULL) {
+			    	char *tok;
+			    	wr_speed += strlen ("Maximum write speed in kB/s:");
+				for (tok = strtok (wr_speed, " (),\t\n");
+					tok && strcmp (tok, "CD");
+					tok = strtok (NULL, " (),\t\n")) {}
+				tok = strtok (NULL, " (),\t\n"); /* Get the CD speed. */
+				*max_wr_speed = atol (tok);
+			}
+		}
+
+		rd_speed = strstr (stdout_data, "Maximum read  speed:");
+		if (rd_speed != NULL) {
+		    	char *tok;
+			rd_speed += strlen ("Maximum read  speed:");
+			for (tok = strtok (rd_speed, " (),\t\n"); 
+				tok && strcmp (tok, "CD"); 
+				tok = strtok (NULL, " (),\t\n")) {}
+			tok = strtok (NULL, " (),\t\n"); /* Get the CD speed. */
+			*max_rd_speed = atol (tok);
+		}
+		else {
+		    	rd_speed = strstr (stdout_data, "Maximum read  speed in kB/s:");
+			if (rd_speed != NULL) {
+			    	char *tok;
+			    	rd_speed += strlen ("Maximum read  speed in kB/s:");
+				for (tok = strtok (rd_speed, " (),\t\n");
+					tok && strcmp (tok, "CD");
+					tok = strtok (NULL, " (),\t\n")) {}
+				tok = strtok (NULL, " (),\t\n"); /* Get the CD speed. */
+				*max_rd_speed = atol (tok);
+			}
+		}
+		drive_cap = strstr (stdout_data, "Does write DVD-RAM media");
+		if (drive_cap != NULL) {
+		    	*type |= CDDRIVE_TYPE_DVD_RAM_RECORDER;
+		}
+		drive_cap = strstr (stdout_data, "Does read DVD-R media");
+		if (drive_cap != NULL) {
+		    	*type |= CDDRIVE_TYPE_DVD_RW_RECORDER;
+		}
+		drive_cap = strstr (stdout_data, "Does read DVD-ROM media");
+		if (drive_cap != NULL) {
+		    	*type |= CDDRIVE_TYPE_DVD_DRIVE;
+		}
+		drive_cap = strstr (stdout_data, "Does write CD-RW media");
+		if (drive_cap != NULL) {
+		    	*type |= CDDRIVE_TYPE_CDRW_RECORDER;
+		}
+		drive_cap = strstr (stdout_data, "Does write CD-R media");
+		if (drive_cap != NULL) {
+		    	*type |= CDDRIVE_TYPE_CD_RECORDER;
+		}
+		drive_cap = strstr (stdout_data, "Does read CD-R media");
+		if (drive_cap != NULL) {
+		    	*type |= CDDRIVE_TYPE_CD_DRIVE;
+		}
+		g_free (stdout_data);
+	}
+
+	g_free (dev_str);
+}
+
 static GList *
 freebsd_scan (gboolean recorder_only)
 {
 	GList *cdroms_list = NULL;
 	const char *dev_type = "cd";
-	int fd;
 	int speed = 16; /* XXX Hardcode the write speed for now. */
-	int max_speed = CDR_MAX_SPEED;
 	int i = 0;
+	int cnode = 1; /* Use the CD device's 'c' node. */
 
 	while (1) {
-		CDDriveType type;
 		CDDrive *cdrom;
-		gchar *cam_path, *dev;
+		gchar *cam_path;
 		struct cam_device *cam_dev;
 
-		type = 0;
 		cam_path = g_strdup_printf ("/dev/%s%dc", dev_type, i);
 
 		if (!g_file_test (cam_path, G_FILE_TEST_EXISTS)) {
 			g_free (cam_path);
-			break;
+			cam_path = g_strdup_printf ("/dev/%s%d", dev_type, i);
+			cnode = 0;
+			if (!g_file_test (cam_path, G_FILE_TEST_EXISTS)) {
+				g_free (cam_path);
+				break;
+			}
 		}
 
 		if ((cam_dev = cam_open_spec_device (dev_type, i, O_RDWR, NULL)) == NULL) {
@@ -788,47 +1081,41 @@ freebsd_scan (gboolean recorder_only)
 			continue;
 		}
 
-		/* XXX Other controllers might need to be added. */
-		if ((strncmp (cam_dev->sim_name, "ata", 3)) == 0) {
-			dev = g_strdup_printf ("/dev/a%s%dc", dev_type, i);
-		} else {
-			dev = g_strdup (cam_path);
-		}
+		cdrom = g_new0 (CDDrive, 1);
+		cdrom->display_name = g_strdup_printf ("%s %s", cam_dev->inq_data.vendor, cam_dev->inq_data.revision);
+		cdrom->device = g_strdup (cam_path);
+		cdrom->cdrecord_id = g_strdup_printf ("%d,%d,%d", cam_dev->path_id, cam_dev->target_id, cam_dev->target_lun);
+		/* Attempt to get more specific information from
+		 * this drive by using cdrecord.
+		 */
+		get_cd_properties (cdrom->cdrecord_id,
+			&(cdrom->max_speed_read),
+			&(cdrom->max_speed_write),
+			&(cdrom->type));
+		if (cdrom->type & CDDRIVE_TYPE_CD_RECORDER
+				|| cdrom->type & CDDRIVE_TYPE_CDRW_RECORDER
+				|| cdrom->type & CDDRIVE_TYPE_DVD_RAM_RECORDER
+				|| cdrom->type & CDDRIVE_TYPE_DVD_RW_RECORDER
+				|| !recorder_only) {
 
-		g_free (cam_path);
+			if (cdrom->max_speed_read == -1) {
+		    		cdrom->max_speed_read = speed;
+			}
+			if (cdrom->max_speed_write == -1) {
+			    	cdrom->max_speed_write = speed;
+			}
 
-		if ((fd = open (dev, O_RDWR, 0)) < 0) {
-			g_free (dev);
-			free (cam_dev);
-			i++;
-			continue;
-		}
-
-		type |= CDDRIVE_TYPE_CD_DRIVE;
-		if (ioctl (fd, CDRIOCWRITESPEED, &max_speed) >= 0) {
-			/* XXX is it CD-R or CD-RW ? */
-			type |= CDDRIVE_TYPE_CD_RECORDER;
-		}
-
-		close (fd);
-
-		if (type & CDDRIVE_TYPE_CD_RECORDER
-				|| type & CDDRIVE_TYPE_CDRW_RECORDER
-				|| recorder_only == FALSE) {
-			cdrom = g_new0 (CDDrive, 1);
-			cdrom->display_name = g_strdup_printf ("%s %s", cam_dev->inq_data.vendor, cam_dev->inq_data.revision);
-			cdrom->device = g_strdup (dev);
-			cdrom->max_speed_read = speed;
-			cdrom->max_speed_write = speed;
-			cdrom->cdrecord_id = g_strdup_printf ("%d,%d,%d", cam_dev->path_id, cam_dev->target_id, cam_dev->target_lun);
-			cdrom->type = type;
-
-			add_dvd_plus (cdrom);
+			if (cdrom->type & CDDRIVE_TYPE_DVD_DRIVE) {
+				add_dvd_plus (cdrom);
+			}
 
 			cdroms_list = g_list_append (cdroms_list, cdrom);
 		}
+		else {
+		    	cd_drive_free (cdrom);
+		}
 
-		g_free (dev);
+		g_free (cam_path);
 		free (cam_dev);
 
 		i++;
@@ -844,13 +1131,19 @@ scan_for_cdroms (gboolean recorder_only, gboolean add_image)
 	CDDrive *cdrom;
 	GList *cdroms = NULL;
 
+#ifdef USE_HAL
+	cdroms = hal_scan (recorder_only);
+#endif
+
+	if (cdroms == NULL) {
 #ifdef __linux__
-	cdroms = linux_scan (recorder_only);
+		cdroms = linux_scan (recorder_only);
 #endif
 
 #ifdef __FreeBSD__
-	cdroms = freebsd_scan (recorder_only);
+		cdroms = freebsd_scan (recorder_only);
 #endif
+	}
 
 	if (add_image) {
 		/* File */
@@ -874,6 +1167,7 @@ cd_drive_free (CDDrive *drive)
 	g_free (drive->display_name);
 	g_free (drive->cdrecord_id);
 	g_free (drive->device);
+	g_free (drive);
 }
 
 CDMediaType
