@@ -56,6 +56,7 @@ struct TotemPlaylistPrivate
 	GtkWidget *treeview;
 	GtkTreeModel *model;
 	GtkTreePath *current;
+	GtkTreeSelection *selection;
 	TotemPlParser *parser;
 
 	/* This is the playing icon */
@@ -197,7 +198,6 @@ totem_playlist_error (char *title, char *reason, TotemPlaylist *playlist)
 				"<b>%s</b>\n%s.", title_esc, reason_esc);
 	g_free (title_esc);
 	g_free (reason_esc);
-	gtk_dialog_set_has_separator (GTK_DIALOG (error_dialog), FALSE);
 	gtk_container_set_border_width (GTK_CONTAINER (error_dialog), 5);
 	gtk_dialog_set_default_response (GTK_DIALOG (error_dialog),
 			GTK_RESPONSE_OK);
@@ -415,6 +415,68 @@ drop_cb (GtkWidget     *widget,
 			NULL);
 }
 
+static void
+on_copy1_activate (GtkButton *button, TotemPlaylist *playlist)
+{
+	GList *l;
+	GtkTreePath *path;
+	GtkClipboard *clip;
+	char *url;
+	GtkTreeIter iter;
+
+	l = gtk_tree_selection_get_selected_rows (playlist->_priv->selection,
+			NULL);
+	path = l->data;
+
+	gtk_tree_model_get_iter (playlist->_priv->model, &iter, path);
+
+	gtk_tree_model_get (playlist->_priv->model,
+			&iter,
+			URI_COL, &url,
+			-1);
+
+	/* Set both the middle-click and the super-paste buffers */
+	clip = gtk_clipboard_get_for_display
+		(gdk_display_get_default(), GDK_SELECTION_CLIPBOARD);
+	gtk_clipboard_set_text (clip, url, -1);
+	clip = gtk_clipboard_get_for_display
+		(gdk_display_get_default(), GDK_SELECTION_PRIMARY);
+	gtk_clipboard_set_text (clip, url, -1);
+	g_free (url);
+
+	g_list_foreach (l, (GFunc) gtk_tree_path_free, NULL);
+	g_list_free (l);
+}
+
+static gboolean
+treeview_button_pressed (GtkTreeView *treeview, GdkEventButton *event,
+		TotemPlaylist *playlist)
+{
+	GtkTreePath *path;
+	GtkWidget *menu;
+
+	if (event->type != GDK_BUTTON_PRESS
+			|| event->button != 3)
+		return FALSE;
+
+	if (gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (treeview),
+				event->x, event->y, &path,
+				NULL, NULL, NULL) == FALSE)
+	{
+		return FALSE;
+	}
+
+	gtk_tree_selection_unselect_all (playlist->_priv->selection);
+	gtk_tree_selection_select_path (playlist->_priv->selection, path);
+	gtk_tree_path_free (path);
+
+	menu = glade_xml_get_widget (playlist->_priv->xml, "menu1");
+	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL,
+			event->button, event->time);
+
+	return TRUE;
+}
+
 static void
 selection_changed (GtkTreeSelection *treeselection, TotemPlaylist *playlist)
 {
@@ -473,11 +535,18 @@ totem_playlist_add_files (GtkWidget *widget, TotemPlaylist *playlist)
 	GtkWidget *fs;
 	int response;
 
-	fs = gtk_file_selection_new (_("Select files"));
-	gtk_file_selection_set_select_multiple (GTK_FILE_SELECTION (fs), TRUE);
+	fs = gtk_file_chooser_dialog_new (_("Select files"),
+			GTK_WINDOW (playlist), GTK_FILE_CHOOSER_ACTION_OPEN,
+			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+			GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
+			NULL);
+	gtk_dialog_set_default_response (GTK_DIALOG (fs), GTK_RESPONSE_ACCEPT);
+	gtk_file_chooser_set_select_multiple (GTK_FILE_CHOOSER (fs), TRUE);
+	gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (fs), FALSE);
+
 	if (playlist->_priv->path != NULL)
 	{
-		gtk_file_selection_set_filename (GTK_FILE_SELECTION (fs),
+		gtk_file_chooser_set_uri (GTK_FILE_CHOOSER (fs),
 				playlist->_priv->path);
 	}
 	response = gtk_dialog_run (GTK_DIALOG (fs));
@@ -485,29 +554,38 @@ totem_playlist_add_files (GtkWidget *widget, TotemPlaylist *playlist)
 	while (gtk_events_pending())
 		gtk_main_iteration();
 
-	if (response == GTK_RESPONSE_OK)
+	if (response == GTK_RESPONSE_ACCEPT)
 	{
-		char **filenames;
-		int i;
+		GSList *filenames, *l;
+		char *mrl;
 
-		filenames = gtk_file_selection_get_selections
-			(GTK_FILE_SELECTION (fs));
+		filenames = gtk_file_chooser_get_uris (GTK_FILE_CHOOSER (fs));
+		if (filenames == NULL)
+		{
+			gtk_widget_destroy (fs);
+			return;
+		}
 
-		if (filenames[0] != NULL)
+		mrl = filenames->data;
+		if (mrl != NULL)
 		{
 			char *tmp;
 
-			tmp = g_path_get_dirname (filenames[0]);
+			tmp = g_path_get_dirname (mrl);
 			g_free (playlist->_priv->path);
 			playlist->_priv->path = g_strconcat (tmp,
 					G_DIR_SEPARATOR_S, NULL);
 			g_free (tmp);
 		}
 
-		for (i = 0; filenames[i] != NULL; i++)
-			totem_playlist_add_mrl (playlist, filenames[i], NULL);
+		for (l = filenames; l != NULL; l = l->next)
+		{
+			mrl = l->data;
+			totem_playlist_add_mrl (playlist, mrl, NULL);
+			g_free (mrl);
+		}
 
-		g_strfreev (filenames);
+		g_slist_free (filenames);
 	}
 
 	gtk_widget_destroy (fs);
@@ -638,10 +716,17 @@ totem_playlist_save_files (GtkWidget *widget, TotemPlaylist *playlist)
 	GtkWidget *fs;
 	int response;
 
-	fs = gtk_file_selection_new (_("Save playlist"));
+	fs = gtk_file_chooser_dialog_new (_("Save playlist"),
+			GTK_WINDOW (playlist), GTK_FILE_CHOOSER_ACTION_SAVE,
+			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+			GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
+			NULL);
+	gtk_dialog_set_default_response (GTK_DIALOG (fs), GTK_RESPONSE_ACCEPT);
+	gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (fs), FALSE);
+
 	if (playlist->_priv->save_path != NULL)
 	{
-		gtk_file_selection_set_filename (GTK_FILE_SELECTION (fs),
+		gtk_file_chooser_set_uri (GTK_FILE_CHOOSER (fs),
 				playlist->_priv->save_path);
 	}
 
@@ -650,22 +735,22 @@ totem_playlist_save_files (GtkWidget *widget, TotemPlaylist *playlist)
 	while (gtk_events_pending())
 		gtk_main_iteration();
 
-	if (response == GTK_RESPONSE_OK)
+	if (response == GTK_RESPONSE_ACCEPT)
 	{
-		const char *filename;
+		char *filename, *tmp;
 
-		filename = gtk_file_selection_get_filename
-			(GTK_FILE_SELECTION (fs));
-		if (filename != NULL)
-		{
-			char *tmp;
+		filename = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (fs));
 
-			tmp = g_path_get_dirname (filename);
-			g_free (playlist->_priv->save_path);
-			playlist->_priv->save_path = g_strconcat (tmp,
-					G_DIR_SEPARATOR_S, NULL);
-			g_free (tmp);
-		}
+		gtk_widget_destroy (fs);
+
+		if (filename == NULL)
+			return;
+
+		tmp = g_path_get_dirname (filename);
+		g_free (playlist->_priv->save_path);
+		playlist->_priv->save_path = g_strconcat (tmp,
+				G_DIR_SEPARATOR_S, NULL);
+		g_free (tmp);
 
 		if (g_file_test (filename, G_FILE_TEST_EXISTS) != FALSE)
 		{
@@ -678,7 +763,6 @@ totem_playlist_save_files (GtkWidget *widget, TotemPlaylist *playlist)
 				 GTK_BUTTONS_NONE,
 				 _("A file named '%s' already exists.\nAre you sure you want to overwrite it?"),
 				 filename);
-			gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
 			gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
 			gtk_dialog_add_buttons (GTK_DIALOG (dialog),
 					GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT,
@@ -689,15 +773,16 @@ totem_playlist_save_files (GtkWidget *widget, TotemPlaylist *playlist)
 			gtk_widget_destroy (dialog);
 			if (response != GTK_RESPONSE_ACCEPT)
 			{
-				gtk_widget_destroy (fs);
+				g_free (filename);
 				return;
 			}
 		}
 
 		totem_playlist_save_current_playlist (playlist, filename);
+		g_free (filename);
+	} else {
+		gtk_widget_destroy (fs);
 	}
-
-	gtk_widget_destroy (fs);
 }
 
 static void
@@ -934,6 +1019,8 @@ init_treeview (GtkWidget *treeview, TotemPlaylist *playlist)
 			G_CALLBACK (selection_changed), playlist);
 	g_signal_connect (G_OBJECT (treeview), "row-activated",
 			G_CALLBACK (treeview_row_changed), playlist);
+	g_signal_connect (G_OBJECT (treeview), "button-press-event",
+			G_CALLBACK (treeview_button_pressed), playlist);
 
 	/* Drag'n'Drop */
 	g_signal_connect (G_OBJECT (treeview), "drag_data_received",
@@ -941,6 +1028,8 @@ init_treeview (GtkWidget *treeview, TotemPlaylist *playlist)
 	gtk_drag_dest_set (treeview, GTK_DEST_DEFAULT_ALL,
 			target_table, 1, GDK_ACTION_COPY);
 
+	playlist->_priv->selection = selection;
+
 	gtk_widget_show (treeview);
 }
 
@@ -1160,7 +1249,6 @@ totem_playlist_init (TotemPlaylist *playlist)
 			G_CALLBACK (totem_playlist_entry_parsed),
 			playlist);
 
-	gtk_dialog_set_has_separator (GTK_DIALOG (playlist), FALSE);
 	gtk_container_set_border_width (GTK_CONTAINER (playlist), 5);
 }
 
@@ -1263,7 +1351,7 @@ totem_playlist_new (const char *glade_filename, GdkPixbuf *playing_pix)
 
 	playlist = TOTEM_PLAYLIST (g_object_new (GTK_TYPE_PLAYLIST, NULL));
 
-	playlist->_priv->xml = glade_xml_new (glade_filename, "vbox4", NULL);
+	playlist->_priv->xml = glade_xml_new (glade_filename, NULL, NULL);
 	if (playlist->_priv->xml == NULL)
 	{
 		totem_playlist_finalize (G_OBJECT (playlist));
@@ -1303,16 +1391,25 @@ totem_playlist_new (const char *glade_filename, GdkPixbuf *playing_pix)
 			G_CALLBACK (totem_playlist_down_files),
 			playlist);
 
+	item = glade_xml_get_widget (playlist->_priv->xml, "copy1");
+	g_signal_connect (G_OBJECT (item), "activate",
+			G_CALLBACK (on_copy1_activate), playlist);
+
 	gtk_widget_add_events (GTK_WIDGET (playlist), GDK_KEY_PRESS_MASK);
 	g_signal_connect (G_OBJECT (playlist), "key_press_event",
 			G_CALLBACK (totem_playlist_key_press), playlist);
 
+	/* Reparent the vbox */
+	item = glade_xml_get_widget (playlist->_priv->xml, "dialog-vbox1");
 	container = glade_xml_get_widget (playlist->_priv->xml, "vbox4");
+	g_object_ref (container);
+	gtk_container_remove (GTK_CONTAINER (item), container);
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (playlist)->vbox),
 			container,
 			TRUE,       /* expand */
 			TRUE,       /* fill */
 			0);         /* padding */
+	g_object_unref (container);
 
 	playlist->_priv->treeview = glade_xml_get_widget
 		(playlist->_priv->xml, "treeview1");
