@@ -29,6 +29,7 @@
 #include "totem-private.h"
 #include "totem-preferences.h"
 #include "bacon-cd-selection.h"
+#include "video-utils.h"
 
 #include "debug.h"
 
@@ -50,7 +51,6 @@ totem_action_info (char *reason, Totem *totem)
 				GTK_MESSAGE_INFO,
 				GTK_BUTTONS_OK,
 				"%s", reason);
-	gtk_dialog_set_has_separator (GTK_DIALOG (error_dialog), FALSE);
 	gtk_container_set_border_width (GTK_CONTAINER (error_dialog), 5);
 	gtk_dialog_set_default_response (GTK_DIALOG (error_dialog),
 			GTK_RESPONSE_OK);
@@ -63,35 +63,6 @@ totem_action_info (char *reason, Totem *totem)
 	gtk_widget_show (error_dialog);
 }
 
-
-static gboolean
-totem_display_is_local (Totem *totem)
-{
-	const char *name, *work;
-	int display, screen;
-
-	name = gdk_display_get_name (gdk_display_get_default ());
-	if (name == NULL)
-		return TRUE;
-
-	work = strstr (name, ":");
-	if (work == NULL)
-		return TRUE;
-
-	/* Get to the character after the colon */
-	work++;
-	if (work == NULL)
-		return TRUE;
-
-	if (sscanf (work, "%d.%d", &display, &screen) != 2)
-		return TRUE;
-
-	if (display < 10)
-		return TRUE;
-
-	return FALSE;
-}
-
 static void
 hide_prefs (GtkWidget *widget, int trash, Totem *totem)
 {
@@ -112,7 +83,6 @@ ask_show_visuals (Totem *totem)
 				_("It seems you are running Totem remotely.\n"
 				"Are you sure you want to enable the visual "
 				"effects?"));
-	gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
 	gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
 	gtk_dialog_set_default_response (GTK_DIALOG (dialog),
 			GTK_RESPONSE_NO);
@@ -177,7 +147,7 @@ on_checkbutton2_toggled (GtkToggleButton *togglebutton, Totem *totem)
 
 	value = gtk_toggle_button_get_active (togglebutton);
 
-	if (value != FALSE && totem_display_is_local (totem) == FALSE)
+	if (value != FALSE && totem_display_is_local () == FALSE)
 	{
 		if (ask_show_visuals (totem) == FALSE)
 		{
@@ -434,6 +404,24 @@ hue_changed (GtkRange *range, Totem *totem)
 			BVW_VIDEO_HUE, (int) i);
 }
 
+static void
+on_tpw_color_reset_clicked (GtkButton *button, Totem *totem)
+{
+	guint i;
+	char *scales[] = {
+		"tpw_bright_scale",
+		"tpw_contrast_scale",
+		"tpw_saturation_scale",
+		"tpw_hue_scale"
+	};
+
+	for (i = 0; i < G_N_ELEMENTS (scales); i++) {
+		GtkWidget *item;
+		item = glade_xml_get_widget (totem->xml, scales[i]);
+		gtk_range_set_value (GTK_RANGE (item), 65535/2);
+	}
+}
+
 static void
 audio_out_menu_changed (GtkOptionMenu *option_menu, Totem *totem)
 {
@@ -455,7 +443,7 @@ totem_setup_preferences (Totem *totem)
 
 	g_return_if_fail (totem->gc != NULL);
 
-	is_local = totem_display_is_local (totem);
+	is_local = totem_display_is_local ();
 
 	gconf_client_add_dir (totem->gc, "/apps/totem",
 			GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
@@ -610,6 +598,11 @@ totem_setup_preferences (Totem *totem)
 	g_signal_connect (G_OBJECT (item), "value-changed",
 			G_CALLBACK (hue_changed), totem);
 
+	/* Reset colour balance */
+	item = glade_xml_get_widget (totem->xml, "tpw_color_reset");
+	g_signal_connect (G_OBJECT (item), "clicked",
+			G_CALLBACK (on_tpw_color_reset_clicked), totem);
+
 	/* Sound output type */
 	item = glade_xml_get_widget (totem->xml, "tpw_sound_output_optionmenu");
 	audio_out = bacon_video_widget_get_audio_out_type (totem->bvw);
@@ -651,8 +644,11 @@ totem_preferences_tvout_setup (Totem *totem)
 	case TV_OUT_NONE:
 		name = "tpw_notvout_radio_button";
 		break;
-	case TV_OUT_TVMODE:
-		name = "tpw_tvoutmode_radio_button";
+	case TV_OUT_NVTV_PAL:
+		name = "tpw_nvtvpalmode_radio_button";
+		break;
+	case TV_OUT_NVTV_NTSC:
+		name = "tpw_nvtvntscmode_radio_button";
 		break;
 	case TV_OUT_DXR3:
 		name = "tpw_dxr3tvout_radio_button";
@@ -670,16 +666,32 @@ totem_preferences_tvout_setup (Totem *totem)
 			G_CALLBACK (on_tvout_toggled), totem);
 	g_object_set_data (G_OBJECT (item), "tvout_type",
 			GINT_TO_POINTER (TV_OUT_NONE));
-	item = glade_xml_get_widget (totem->xml, "tpw_tvoutmode_radio_button");
+	gtk_widget_set_sensitive(item, 
+			bacon_video_widget_fullscreen_mode_available (totem->bvw, TV_OUT_NONE));
+
+	item = glade_xml_get_widget (totem->xml, "tpw_nvtvpalmode_radio_button");
+        g_signal_connect (G_OBJECT (item), "toggled",
+                G_CALLBACK (on_tvout_toggled), totem);
+        g_object_set_data (G_OBJECT (item), "tvout_type",
+                GINT_TO_POINTER (TV_OUT_NVTV_PAL));
+	gtk_widget_set_sensitive(item, 
+			bacon_video_widget_fullscreen_mode_available (totem->bvw, TV_OUT_NVTV_PAL));
+
+	item = glade_xml_get_widget (totem->xml, "tpw_nvtvntscmode_radio_button");
 	g_signal_connect (G_OBJECT (item), "toggled",
 			G_CALLBACK (on_tvout_toggled), totem);
 	g_object_set_data (G_OBJECT (item), "tvout_type",
-			GINT_TO_POINTER (TV_OUT_TVMODE));
+                GINT_TO_POINTER (TV_OUT_NVTV_NTSC));
+	gtk_widget_set_sensitive(item, 
+			bacon_video_widget_fullscreen_mode_available (totem->bvw, TV_OUT_NVTV_NTSC));
+	
 	item = glade_xml_get_widget (totem->xml, "tpw_dxr3tvout_radio_button");
 	g_signal_connect (G_OBJECT (item), "toggled",
 			G_CALLBACK (on_tvout_toggled), totem);
 	g_object_set_data (G_OBJECT (item), "tvout_type",
 			GINT_TO_POINTER (TV_OUT_DXR3));
+	gtk_widget_set_sensitive(item, 
+			bacon_video_widget_fullscreen_mode_available (totem->bvw, TV_OUT_DXR3));
 }
 
 void
