@@ -6,6 +6,8 @@
 #include <X11/Xlib.h>
 #include <stdlib.h>
 #include <unistd.h>
+#include <string.h>
+#include <stdio.h>
 
 /* Code taken from:
  * transcode Copyright (C) Thomas Oestreich - June 2001
@@ -210,3 +212,29 @@ void totem_create_symlinks (const char *orig, const char *dest)
 	g_dir_close (dir);
 }
 
+gboolean totem_display_is_local (void)
+{
+	const char *name, *work;
+	int display, screen;
+
+	name = gdk_display_get_name (gdk_display_get_default ());
+	if (name == NULL)
+		return TRUE;
+
+	work = strstr (name, ":");
+	if (work == NULL)
+		return TRUE;
+
+	/* Get to the character after the colon */
+	work++;
+	if (work == NULL)
+		return TRUE;
+
+	if (sscanf (work, "%d.%d", &display, &screen) != 2)
+		return TRUE;
+
+	if (display < 10)
+		return TRUE;
+
+	return FALSE;
+}
